<?php
/**
* NOTICE OF LICENSE
*
*   This file is property of Petr Hucik. You may NOT redistribute the code in any way
*   See license.txt for the complete license agreement
*
* @author    Petr Hucik
* @website   https://www.getdatakick.com
* @copyright Petr Hucik <petr@getdatakick.com>
* @license   see license.txt
* @version   2.1.0
*/
namespace Datakick;

class ProductPriceFunction extends Func {
    public function __construct() {
      parent::__construct('productPrice', 'currency', array(
        'names' => array('productId', 'attributeId', 'useTax'),
        'types' => array('number', 'number', 'boolean')
      ), false);
    }

    public function evaluate($args, $argsTypes, Context $context) {
        $shop = new \Shop($context->getValue('shop'));
        $currency = new \Currency($context->getValue('defaultCurrency'));
        $psContext = \Context::getContext()->cloneContext();
        $psContext->shop = $shop;
        $psContext->currency = $currency;
        return self::calculatePrice($psContext, $args[0], $args[1], $args[2]);
    }

    public static function calculatePrice($psContext, $productId, $attrId, $usetax) {
        $value;
        $spec = null;
        if (! is_null($productId)) {
          $value = \Product::getPriceStatic($productId, $usetax, $attrId, 6, null, false, true, 1, false, null, null, null, $spec, true, true, $psContext, true);
        }
        return new Currency($psContext->currency->id, $value);
    }

    public function jsEvaluate() {
      return 'return 1;';
    }

    public function getExtractor($childExtractors, $childTypes, Context $context, $query, $factory) {
      return new ProductPriceExtractor($childExtractors, $context);
    }
}


class ProductPriceExtractor extends Extractor {
    private $extractors;

    public function __construct($extractors, Context $context) {
        $this->extractors = $extractors;
        $shop = new \Shop($context->getValue('shop'));
        $currency = new \Currency($context->getValue('defaultCurrency'));

        $this->psContext = \Context::getContext();
        $this->psContext->shop = $shop;
        $this->psContext->currency = $currency;
    }

    public function getValue($resultset) {
        $productId = $this->extractors[0]->getValue($resultset);
        $attrId = $this->extractors[1]->getValue($resultset);
        $usetax = $this->extractors[2]->getValue($resultset);
        return ProductPriceFunction::calculatePrice($this->psContext, $productId, $attrId, $usetax);
    }
}
