<?php
/**
* NOTICE OF LICENSE
*
*   This file is property of Petr Hucik. You may NOT redistribute the code in any way
*   See license.txt for the complete license agreement
*
* @author    Petr Hucik
* @website   https://www.getdatakick.com
* @copyright Petr Hucik <petr@getdatakick.com>
* @license   see license.txt
* @version   2.1.0
*/
namespace Datakick\Schema\Prestashop;

class Customer {
  public function register($dictionary) {
    $db = \DB::getInstance();
    $hideB2B = $db->getValue("SELECT IFNULL((SELECT 0 FROM "._DB_PREFIX_."configuration where name = 'PS_B2B_ENABLE' and value='1'), 1)");

    $dictionary->registerCollection(array(
      'id' => 'customers',
      'singular' => 'customer',
      'description' => 'Customers',
      'parameters' => array('shop', 'language', 'shopGroup', 'shareCustomers', 'defaultCurrency'),
      'key' => array('id'),
      'display' => 'fullname',
      'category' => 'relationships',
      'priority' => 300,
      'psTab' => 'AdminCustomers',
      'psController' => 'AdminCustomers',
      'psClass' => 'Customer',
      'restrictions' => array(
        'shop' => array(
          'shop' => '<field:shopId>'
        )
      ),
      'tables' => array(
        'c' => array(
          'table' => 'customer',
          'conditions' => array(
            'if(<bind-param:shareCustomers:1>, <bind-param:shopGroup:c.id_shop_group>, <bind-param:shop:c.id_shop>)'
          )
        ),
        'g' => array(
          'table' => 'gender',
          'require' => array('c'),
          'join' => array(
            'type' => 'LEFT',
            'conditions' => array(
              'c.id_gender = g.id_gender'
            )
          )
        ),
        'gl' => array(
          'table' => 'gender_lang',
          'require' => array('g'),
          'join' => array(
            'type' => 'INNER',
            'conditions' => array(
              'g.id_gender = g.id_gender',
              '<bind-param:language:gl.id_lang>'
            )
          )
        ),
        'r' => array(
          'table' => 'risk',
          'require' => array('c'),
          'join' => array(
            'type' => 'LEFT',
            'conditions' => array(
              'c.id_risk = r.id_risk'
            )
          )
        ),
        'rl' => array(
          'table' => 'risk_lang',
          'require' => array('r'),
          'join' => array(
            'type' => 'LEFT',
            'conditions' => array(
              'r.id_risk = rl.id_risk',
              '<bind-param:language:rl.id_lang>'
            )
          )
        ),
        's' => array(
          'table' => 'shop',
          'require' => array('c'),
          'join' => array(
            'type' => 'INNER',
            'conditions' => array(
              's.id_shop = c.id_shop'
            )
          )
        )
      ),
      'fields' => array(
        'id' => array(
          'type' => 'number',
          'description' => 'id',
          'sql' => 'c.id_customer',
          'require' => array('c'),
          'selectRecord' => 'customers',
          'update' => false
        ),
        'shopId' => array(
          'type' => 'number',
          'description' => 'shop id',
          'sql' => 'c.id_shop',
          'require' => array('c'),
          'hidden' => true,
          'update' => false
        ),
        'shop' => array(
          'type' => 'string',
          'description' => 'shop',
          'sql' => 's.name',
          'require' => array('s'),
          'hidden' => ! SchemaUtils::isMultiShop(),
          'update' => false
        ),
        'defaultGroupId' => array(
          'type' => 'number',
          'description' => 'default group id',
          'sql' => 'c.id_default_group',
          'require' => array('c'),
          'selectRecord' => 'customerGroups',
          'update' => array(
            'c' => 'id_default_group'
          )
        ),
        'fullname' => array(
          'type' => 'string',
          'description' => 'name',
          'sql' => "TRIM(CONCAT(c.firstname, ' ', c.lastname))",
          'require' => array('c'),
          'update' => false
        ),
        'title' => array(
          'type' => 'string',
          'description' => 'title',
          'sql' => 'gl.name',
          'require' => array('gl'),
          'update' => false
        ),
        'firstname' => array(
          'type' => 'string',
          'description' => 'first name',
          'sql' => "TRIM(c.firstname)",
          'require' => array('c'),
          'update' => array(
            'c' => 'firstname'
          )
        ),
        'lastname' => array(
          'type' => 'string',
          'description' => 'last name',
          'sql' => "TRIM(c.lastname)",
          'require' => array('c'),
          'update' => array(
            'c' => 'lastname'
          )
        ),
        'email' => array(
          'type' => 'string',
          'description' => 'email',
          'sql' => "c.email",
          'require' => array('c'),
          'update' => array(
            'c' => 'email'
          )
        ),
        'birthday' => array(
          'type' => 'datetime',
          'description' => 'birthday',
          'sql' => SchemaUtils::normalizeDate('c.birthday'),
          'require' => array('c'),
          'update' => array(
            'c' => 'birthday'
          )
        ),
        'active' => array(
          'type' => 'boolean',
          'description' => 'is enabled',
          'sql' => 'c.active',
          'require' => array('c'),
          'update' => array(
            'c' => 'active'
          )
        ),
        'deleted' => array(
          'type' => 'boolean',
          'description' => 'is deleted',
          'sql' => 'c.deleted',
          'require' => array('c'),
          'update' => array(
            'c' => 'deleted'
          )
        ),
        'newsletter' => array(
          'type' => 'boolean',
          'description' => 'newsletter',
          'sql' => 'c.newsletter',
          'require' => array('c'),
          'update' => array(
            'c' => 'newsletter'
          )
        ),
        'optIn' => array(
          'type' => 'boolean',
          'description' => 'Opt-in',
          'sql' => 'c.optin',
          'require' => array('c'),
          'update' => array(
            'c' => 'optin'
          )
        ),
        'gender' => array(
          'type' => 'string',
          'description' => 'gender',
          'sql' => "IF(g.type=0, 'M', IF(g.type=1, 'F', IF(g.type=2, 'N', '')))",
          'require' => array('g'),
          'update' => array()
        ),
        'created' => array(
          'type' => 'datetime',
          'description' => 'date created',
          'sql' => 'c.date_add',
          'require' => array('c'),
          'update' => false
        ),
        'updated' => array(
          'type' => 'datetime',
          'description' => 'date updated',
          'sql' => 'c.date_upd',
          'require' => array('c'),
          'update' => false
        ),
        'lastVisit' => array(
          'type' => 'datetime',
          'description' => 'last visit',
          'sql' => '(SELECT con.date_add FROM '._DB_PREFIX_.'guest guest LEFT JOIN '._DB_PREFIX_.'connections con ON con.id_guest = guest.id_guest WHERE guest.id_customer = c.id_customer ORDER BY con.date_add DESC LIMIT 1)',
          'require' => array('c'),
          'update' => false
        ),
        // B2B feature on
        'company' => array(
          'type' => 'string',
          'description' => 'company',
          'sql' => 'c.company',
          'require' => array('c'),
          'hidden' => $hideB2B,
          'update' => array(
            'c' => 'company'
          )
        ),
        'siret' => array(
          'type' => 'string',
          'description' => 'SIRET',
          'sql' => 'c.siret',
          'require' => array('c'),
          'hidden' => $hideB2B,
          'update' => array(
            'c' => 'siret'
          )
        ),
        'ape' => array(
          'type' => 'string',
          'description' => 'APE',
          'sql' => 'c.ape',
          'require' => array('c'),
          'hidden' => $hideB2B,
          'update' => array(
            'c' => 'ape'
          )
        ),
        'website' => array(
          'type' => 'string',
          'description' => 'website',
          'sql' => 'c.website',
          'require' => array('c'),
          'hidden' => $hideB2B,
          'update' => array(
            'c' => 'website'
          )
        ),
        'allowOutstanding' => array(
          'type' => 'currency',
          'description' => 'allowed outstanding amount',
          'sql' => array(
            'value' => 'c.outstanding_allow_amount',
            'currency' => '<param:defaultCurrency>'
          ),
          'require' => array('c'),
          'hidden' => $hideB2B,
          'fixedCurrency' => true,
          'update' => array(
            'c' => array(
              'field' => array(
                'value' => 'outstanding_allow_amount'
              )
            )
          )
        ),
        'paymentDays' => array(
          'type' => 'number',
          'description' => 'max payment days',
          'sql' => 'c.max_payment_days',
          'require' => array('c'),
          'hidden' => $hideB2B,
          'update' => array(
            'c' => 'max_payment_days'
          )
        ),
        'risk' => array(
          'type' => 'string',
          'description' => 'risk',
          'sql' => 'coalesce(rl.name, r.percent)',
          'require' => array('rl', 'r'),
          'hidden' => $hideB2B,
          'update' => false
        ),
        'riskPercent' => array(
          'type' => 'number',
          'description' => 'risk %',
          'sql' => 'r.percent',
          'require' => array('r'),
          'hidden' => $hideB2B,
          'update' => false
        ),
      ),
      'expressions' => array(
        'age' => array(
          'type' => 'number',
          'expression' => 'floor(dateDiff(now(), <field:birthday>) / 365)',
          'description' => 'age'
        )
      ),
      'links' => array(
        'defaultGroup' => array(
          'description' => "Default group",
          'collection' => 'customerGroups',
          'type' => 'HAS_ONE',
          'sourceFields' => array('defaultGroupId'),
          'targetFields' => array('id')
        ),
        'groups' => array(
          'description' => "Customer groups",
          'collection' => 'customerGroups',
          'type' => 'HABTM',
          'sourceFields' => array('id'),
          'targetFields' => array('id'),
          'joinTable' => 'customer_group',
          'joinFields' => array(
            'sourceFields' => array('id_customer'),
            'targetFields' => array('id_group'),
          ),
        ),
        'addresses' => array(
          'description' => "Addresses",
          'collection' => 'addresses',
          'type' => 'HAS_MANY',
          'sourceFields' => array('id'),
          'targetFields' => array('customerId')
        ),
        'orders' => array(
          'description' => "Orders",
          'collection' => 'orders',
          'type' => 'HAS_MANY',
          'sourceFields' => array('id'),
          'targetFields' => array('customerId')
        ),
        'carts' => array(
          'description' => "Carts",
          'collection' => 'carts',
          'type' => 'HAS_MANY',
          'sourceFields' => array('id'),
          'targetFields' => array('customerId')
        )
      )
    ));
  }
}
